#ifndef NPSTAT_BINDKERNEL_HH_
#define NPSTAT_BINDKERNEL_HH_

/*!
// \file BindKernel.hh
//
// \brief Special bind operation for two-argument functors which also
//        converts them into Functor1<long double,long double>
//
// Author: I. Volobouev
//
// December 2022
*/

#include "npstat/nm/SimpleFunctors.hh"

namespace npstat {
    template <class Kernel>
    class BindKernelXHelper : public npstat::Functor1<long double,long double>
    {
    public:
        inline BindKernelXHelper(const Kernel& kernel, const long double x)
            : kernel_(kernel), x_(x) {}

        inline virtual ~BindKernelXHelper() {}
        
        inline virtual long double operator()(const long double& y) const
            {return kernel_(x_, y);}

    private:
        const Kernel& kernel_;
        long double x_;
    };

    // Note that the lifetime of the argument kernel should exceed
    // the lifetime of the object produced by this function
    template <class Kernel>
    inline BindKernelXHelper<Kernel> BindKernelX(const Kernel& kernel,
                                                 const long double x)
    {
        return BindKernelXHelper<Kernel>(kernel, x);
    }

    template <class Kernel>
    class BindKernelYHelper : public npstat::Functor1<long double,long double>
    {
    public:
        inline BindKernelYHelper(const Kernel& kernel, const long double y)
            : kernel_(kernel), y_(y) {}

        inline virtual ~BindKernelYHelper() {}

        inline virtual long double operator()(const long double& x) const
            {return kernel_(x, y_);}

    private:
        const Kernel& kernel_;
        long double y_;
    };

    // Note that the lifetime of the argument kernel should exceed
    // the lifetime of the object produced by this function
    template <class Kernel>
    inline BindKernelYHelper<Kernel> BindKernelY(const Kernel& kernel,
                                                 const long double y)
    {
        return BindKernelYHelper<Kernel>(kernel, y);
    }
}

#endif // NPSTAT_BINDKERNEL_HH_
