#ifndef NPSTAT_GAUSSLEGENDREQUADRATURE2D_HH_
#define NPSTAT_GAUSSLEGENDREQUADRATURE2D_HH_

/*!
// \file GaussLegendreQuadrature2D.hh
//
// \brief Tensor-product Gauss-Legendre quadratures in long double precision
//        on rectangular regions
//
// Author: I. Volobouev
//
// October 2020
*/

#include <vector>
#include <utility>

namespace npstat {
    class GaussLegendreQuadrature2D
    {
    public:
        /**
        // The number of supported points in each dimension
        // is the same as for the 1-d Gauss-Legendre quadrature
        */
        GaussLegendreQuadrature2D(unsigned npoints0, unsigned npoints1);

        /** Return the number of quadrature points in the given dimension */
        unsigned npoints(unsigned dim) const;

        /** Perform the quadrature on the [a, b] x [c, d] rectangle */
        template <class Functor>
        long double integrate(const Functor& fcn,
                              long double a, long double b,
                              long double c, long double d) const;
    private:
        std::vector<long double> a0_;
        std::vector<long double> w0_;
        std::vector<long double> a1_;
        std::vector<long double> w1_;
        unsigned npoints0_;
        unsigned npoints1_;
        mutable std::vector<std::pair<long double, long double> > buf_;

#ifdef SWIG
    public:
        template <class Functor>
        inline double integrate2(const Functor& fcn,
                                 const double a, const double b,
                                 const double c, const double d) const
        {
            return integrate(fcn, a, b, c, d);
        }
#endif // SWIG
    };
}

#include "npstat/nm/GaussLegendreQuadrature2D.icc"

#endif // NPSTAT_GAUSSLEGENDREQUADRATURE2D_HH_
