#ifndef NPSTAT_PAIRCOMPAREQ_HH_
#define NPSTAT_PAIRCOMPAREQ_HH_

/*!
// \file PairCompareQ.hh
//
// \brief Comparison functors for quadruple precision pairs
//
// Author: I. Volobouev
//
// July 2023
*/

#include "npstat/nm/quad_float.hh"

namespace npstat {
    template <typename T>
    struct LessByAbsFirstQ
    {
        inline bool operator()(const T& x, const T& y) const
            {return fabsq(x.first) < fabsq(y.first);}
        inline bool operator()(const T* x, const T* y) const
            {return fabsq(x->first) < fabsq(y->first);}
    };

    /** "greater" functor to compare the magnitude of the first element only */
    template <typename T>
    struct GreaterByAbsFirstQ
    {
        inline bool operator()(const T& x, const T& y) const
            {return fabsq(y.first) < fabsq(x.first);}
        inline bool operator()(const T* x, const T* y) const
            {return fabsq(y->first) < fabsq(x->first);}
    };
}

#endif // NPSTAT_PAIRCOMPAREQ_HH_
