#ifndef NPSTAT_RECURRENCECOEFFSQ_HH_
#define NPSTAT_RECURRENCECOEFFSQ_HH_

//======================================================================
// RecurrenceCoeffsQ.hh
//
// This is an internal header which is subject to change without
// notice. Application code should never use classes or functions
// declared/defined in this header directly.
//
// Author: I. Volobouev
//
// July 2023
//======================================================================

#include <vector>
#include <utility>

#include "npstat/nm/RecurrenceQ.hh"
#include "npstat/nm/OrthoPolyMethod.hh"

namespace npstat {
    namespace Private {
        // The following function should be the only one
        // directly used by the rest of the package
        template <class MeasurePt>
        void calcRecurrenceCoeffsQ(const std::vector<MeasurePt>& measure,
                                   unsigned maxdeg, quad_float norm,
                                   OrthoPolyMethod m,
                                   std::vector<RecurrenceQ>* rcoeffs);

        template <class MeasurePt>
        void calcRecurrenceStieltjesQ(const std::vector<MeasurePt>& measure,
                                     unsigned maxdeg, quad_float norm,
                                     std::vector<RecurrenceQ>* rcoeffs);

        template <class MeasurePt>
        void calcRecurrenceLanczosQ(const std::vector<MeasurePt>& measure,
                                   unsigned maxdeg, quad_float norm,
                                   std::vector<RecurrenceQ>* rcoeffs);

        template <class MeasurePt>
        std::pair<quad_float,quad_float> monicInnerProductsQ(
            const std::vector<RecurrenceQ>& rcoeffs,
            const std::vector<MeasurePt>& measure,
            unsigned degree, quad_float norm);

        quad_float monicpolyQ(const std::vector<RecurrenceQ>& rcoeffs,
                               unsigned degree, quad_float x);
    }
}

#include "npstat/nm/RecurrenceCoeffsQ.icc"

#endif // NPSTAT_RECURRENCECOEFFSQ_HH_
