#ifndef NPSTAT_RECURRENCECOEFFSTRGAUSS_HH_
#define NPSTAT_RECURRENCECOEFFSTRGAUSS_HH_

//======================================================================
// RecurrenceCoeffsTrGauss.hh
//
// This is an internal header which is subject to change without
// notice. Application code should never use classes or functions
// declared/defined in this header directly.
//
// Author: I. Volobouev
//
// May 2017
//======================================================================

#include "npstat/nm/RecurrenceCoeffs.hh"
#include "npstat/nm/SimpleFunctors.hh"

namespace npstat {
    namespace Private {
        // "transformFromGauss" is the transform connecting the standard
        // normal variable with the variable distributed with the density
        // equal to the OPS weight.
        //
        // The "Numeric" type should be either double or, preferably,
        // long double.
        //
        // The "quadraturePoints" argument will be passed to
        // GaussHermiteQuadrature and should be supported by that
        // class. The choice of the quadrature rule is a compromise
        // between good precision (more points are better)
        // and the chance that we will exceed the range of
        // double precision numbers when the functor is evaluated
        // (quadratures with more points use larger arguments).
        //
        template<typename Numeric>
        void calcRecurrenceCoeffsTrGauss(
            const Functor1<Numeric, Numeric>& transformFromGauss,
            unsigned maxdeg, OrthoPolyMethod m, unsigned quadraturePoints,
            std::vector<Recurrence>* rcoeffs);
    }
}

#include "npstat/nm/RecurrenceCoeffsTrGauss.icc"

#endif // NPSTAT_RECURRENCECOEFFSTRGAUSS_HH_
