#ifndef NPSTAT_SIMPLESCALARPRODUCT_HH_
#define NPSTAT_SIMPLESCALARPRODUCT_HH_

/*!
// \file SimpleScalarProduct.hh
//
// \brief A simple code to calculate scalar products with unit weight function
//
// Author: I. Volobouev
//
// February 2023
*/

#include <cassert>

#include "npstat/nm/PreciseType.hh"

namespace npstat {
    template<typename Real>
    struct SimpleScalarProduct
    {
        inline Real operator()(const Real* x, const Real* y,
                               const unsigned long len) const
        {
            typedef typename PreciseType<Real>::type Precise;
            Precise sum = Precise();
            if (len)
            {
                assert(x);
                assert(y);
                for (unsigned long i=0; i<len; ++i)
                    sum += static_cast<Precise>(*x++ * *y++);
            }
            return static_cast<Real>(sum);
        }
    };
}

#endif // NPSTAT_SIMPLESCALARPRODUCT_HH_
