#ifndef NPSTAT_TRUNCATEDLOG_HH_
#define NPSTAT_TRUNCATEDLOG_HH_

/*!
// \file TruncatedLog.hh
//
// \brief Truncated log function useful in maximum likelihood fitting
//        of densities for which positivity condition is difficult to impose
//
// Author: I. Volobouev
//
// March 2023
*/

#include "npstat/nm/Poly1D.hh"

namespace npstat {
    template<class Real>
    class TruncatedLog
    {
    public:
        // Note that, typically, the argument "nTaylorTerms"
        // should be odd. Otherwise, eventually, at large in
        // magnitude negative values of x, the expansion will
        // become positive.
        TruncatedLog(Real truncationX, unsigned nTaylorTerms);

        inline Real truncationX() const {return tr_;}
        inline unsigned nTaylorTerms() const
            {return expansion_.deg()+1U;}

        Real operator()(Real x) const;
        Real derivative(Real x) const;
        Real secondDerivative(Real x) const;

        // The following returns the integral from 1 to x
        Real integral(Real x) const;

    private:
        long double ltr_;
        Poly1D expansion_;
        Poly1D deriv1_;
        Poly1D deriv2_;
        Poly1D integ_;
        Real one_;
        Real tr_;
        Real deltaMax_;
    };
}

#include "npstat/nm/TruncatedLog.icc"

#endif // NPSTAT_TRUNCATEDLOG_HH_
