#ifndef NPSTAT_MAXABSVALUE_HH_
#define NPSTAT_MAXABSVALUE_HH_

/*!
// \file maxAbsValue.hh
//
// \brief Maximum absolute value in an array
//
// Author: I. Volobouev
//
// March 2023
*/

#include <cassert>
#include <stdexcept>
#include <cmath>
#include <cstdlib>

namespace npstat {
    template<typename T>
    inline auto maxAbsValue(
        const T* x, const unsigned long len) -> decltype(std::abs(*x))
    {
        typedef decltype(std::abs(*x)) return_type;

        if (!len) throw std::invalid_argument(
            "In npstat::maxAbsValue: empty input array");
        assert(x);
        return_type maxValue = return_type();
        for (unsigned long i=0; i<len; ++i)
        {
            const return_type v = std::abs(*x++);
            if (maxValue < v)
                maxValue = v;
        }
        return maxValue;
    }
}

#endif // NPSTAT_MAXABSVALUE_HH_
