#ifndef NPSTAT_MULTIVARIATECOSINE_HH_
#define NPSTAT_MULTIVARIATECOSINE_HH_

/*!
// \file multivariateCosine.hh
//
// \brief Calculate scalar products of vectors whose length is normalized
//        internally
//
// Author: I. Volobouev
//
// February 2023
*/

#include <cmath>
#include <cassert>
#include <stdexcept>

namespace npstat {
    template<typename Real>
    long double multivariateCosine(const Real* x, const unsigned long strideX,
                                   const Real* y, const unsigned long strideY,
                                   const unsigned long len)
    {
        assert(len);
        assert(x);
        assert(y);

        long double sumX = 0.0L, sumY = 0.0L, sumSprod = 0.0L;
        for (unsigned long i=0; i<len; ++i, x+=strideX, y+=strideY)
        {
            sumX += *x * *x;
            sumSprod += *x * *y;
            sumY += *y * *y;
        }
        if (sumX == 0.0L || sumY == 0.0L) throw std::domain_error(
            "In npstat::multivariateCosine: zero vector norm encountered");
        long double c = sumSprod/sqrtl(sumX)/sqrtl(sumY);
        if (c < -1.0L)
            c = -1.0L;
        if (c > 1.0L)
            c = 1.0L;
        return c;
    }
}

#endif // NPSTAT_MULTIVARIATECOSINE_HH_
