#ifndef NPSTAT_SCALESFROMHESSIAN_HH_
#define NPSTAT_SCALESFROMHESSIAN_HH_

/*!
// \file scalesFromHessian.hh
//
// \brief Make a guess about local function scales from the function Hessian
//
// Author: I. Volobouev
//
// March 2023
*/

#include "npstat/nm/Matrix.hh"

namespace npstat {
    /**
    // Make a guess about typical scales in various directions
    // by looking at the Hessian diagonal elements. The Hessian
    // matrix must be square, and the size of the "scales" array
    // must be at least as large as the number of hessian rows.
    //
    // The diagonal elements of the Hessian are expected to be
    // mostly positive. If they are negative, the "negativeScaleLimit"
    // limit (on the maximum scale) will be imposed on them.
    */
    void scalesFromHessian(const Matrix<double>& hessian,
                           double negativeScaleLimit,
                           double* scales, unsigned nScales);

    /**
    // Make a guess about the covariance matrix.
    // This will be just the inverse of the Hessian
    // in case the Hessian is positive-definite.
    */
    Matrix<double> covmatFromHessian(const Matrix<double>& hessian,
                                     double negativeScaleLimit);
}

#endif // NPSTAT_SCALESFROMHESSIAN_HH_
