#ifndef NPSTAT_SHIFTEDLEGENDREINTEGRAL_HH_
#define NPSTAT_SHIFTEDLEGENDREINTEGRAL_HH_

/*!
// \file shiftedLegendreIntegral.hh
//
// \brief Integrals between 0 and 1 of products of shifted Legendre
//        polynomials orthonormal on [0, 1]
//
// Author: I. Volobouev
//
// July 2023
*/

namespace npstat {
    // In the subsequent declarations, i, j, k, and l are
    // the polynomial degrees
    inline long double shiftedLegendreIntegral(const unsigned i)
        {return i ? 0.0L : 1.0L;}

    inline long double shiftedLegendreIntegral(const unsigned i,
                                               const unsigned j)
        {return i == j ? 1.0L : 0.0L;}

    long double shiftedLegendreIntegral(unsigned i, unsigned j, unsigned k);

    long double shiftedLegendreIntegral(unsigned i, unsigned j,
                                        unsigned k, unsigned l,
                                        bool memoize = false);
}

#endif // NPSTAT_SHIFTEDLEGENDREINTEGRAL_HH_
