#ifndef NPSTAT_UNITMATRIXDEVIATIONS_HH_
#define NPSTAT_UNITMATRIXDEVIATIONS_HH_

/*!
// \file unitMatrixDeviations.hh
//
// \brief Routines for finding deviations from unit matrix behavior
//        as a function of submatrix size
//
// Author: I. Volobouev
//
// June 2023
*/

#include "npstat/nm/Matrix.hh"

namespace npstat {
    //@{
    /**
    // The following function returns the size of the maximum absolute
    // deviation as a function of leading principal submatrix size.
    // result[0] will contain the deviation for 1 x 1 submatrix,
    // result[1] will contain the deviation for 2 x 2 submatrix, etc.
    */
    template<typename Numeric, unsigned Len, typename T>
    void unitMatrixDeviations(const Matrix<Numeric,Len>& mat,
                              std::vector<T>* result);
    template<typename Numeric, unsigned Len, typename T>
    void unitMatrixDeviationsFrobenius(const Matrix<Numeric,Len>& mat,
                                       std::vector<T>* result);
    //@}

    //@{
    /**
    // The following function returns the size of the leading principal
    // submatrix whose absolute deviations from the unit matrix do not
    // exceeed the requested tolerance.
    */
    template<typename Numeric, unsigned Len>
    unsigned deviationsRestrictedSize(const Matrix<Numeric,Len>& mat,
                                      double tolerance);
    template<typename Numeric, unsigned Len>
    unsigned deviationsRestrictedSizeFrobenius(const Matrix<Numeric,Len>& mat,
                                               double tolerance);
    //@}
}

#include "npstat/nm/unitMatrixDeviations.icc"

#endif // NPSTAT_UNITMATRIXDEVIATIONS_HH_
