#include "npstat/stat/Distribution1DReader.hh"

#include "npstat/stat/Distributions1D.hh"
#include "npstat/stat/JohnsonCurves.hh"
#include "npstat/stat/CompositeDistribution1D.hh"
#include "npstat/stat/GaussianMixture1D.hh"
#include "npstat/stat/DistributionMix1D.hh"
#include "npstat/stat/TruncatedDistribution1D.hh"
#include "npstat/stat/FoldedDistribution1D.hh"
#include "npstat/stat/QuantileTable1D.hh"
#include "npstat/stat/LeftCensoredDistribution.hh"
#include "npstat/stat/RightCensoredDistribution.hh"
#include "npstat/stat/RatioOfNormals.hh"
#include "npstat/stat/UGaussConvolution1D.hh"
#include "npstat/stat/InterpolatedDistro1D1P.hh"
#include "npstat/stat/InterpolatedDistro1DNP.hh"
#include "npstat/stat/TransformedDistribution1D.hh"
#include "npstat/stat/LocationScaleFamily1D.hh"
#include "npstat/stat/EdgeworthSeries1D.hh"
#include "npstat/stat/DeltaMixture1D.hh"
#include "npstat/stat/ComparisonDistribution1D.hh"
#include "npstat/stat/PolynomialDistro1D.hh"
#include "npstat/stat/LegendreDistro1D.hh"
#include "npstat/stat/ChebyshevDistro1D.hh"
#include "npstat/stat/PowerHalfCauchy1D.hh"
#include "npstat/stat/PowerRatio1D.hh"

#define add_reader(Derived) do {\
     const gs::ClassId& id(gs::ClassId::makeId<Derived >());\
     (*this)[id.name()] = new gs::ConcreteReader<AbsDistribution1D,Derived >();\
} while(0);

namespace npstat {
    Distribution1DReader::Distribution1DReader()
    {
        // Distributions inheriting from AbsScalableDistribution1D
        // This list and the list in "Distribution1DFactory.cc"
        // should be similar.
        add_reader(Beta1D);
        add_reader(BifurcatedGauss1D);
        add_reader(BinnedDensity1D);
        add_reader(Cauchy1D);
        add_reader(DeltaMixture1D);
        add_reader(Exponential1D);
        add_reader(Laplace1D);
        add_reader(Gamma1D);
        add_reader(Gauss1D);
        add_reader(GaussianMixture1D);
        add_reader(Huber1D);
        add_reader(IsoscelesTriangle1D);
        add_reader(JohnsonSb);
        add_reader(JohnsonSu);
        add_reader(JohnsonSystem);
        add_reader(LocationScaleFamily1D);
        add_reader(Logistic1D);
        add_reader(LogNormal);
        add_reader(LogQuadratic1D);
        add_reader(MirroredGauss1D);
        add_reader(Moyal1D);
        add_reader(Pareto1D);
        add_reader(Quadratic1D);
        add_reader(QuantileTable1D);
        add_reader(StudentsT1D);
        add_reader(SymmetricBeta1D);
        add_reader(Tabulated1D);
        add_reader(TruncatedGauss1D);
        add_reader(UGaussConvolution1D);
        add_reader(Uniform1D);
        add_reader(UniPareto1D);
        add_reader(PolynomialDistro1D);
        add_reader(LegendreDistro1D);
        add_reader(ChebyshevDistro1D);
        add_reader(PowerHalfCauchy1D);
        add_reader(PowerRatio1D);

        // We are not going to add here AbsInterpolatedDistribution1D
        // and its descendants. This is because distributions of this
        // kind represent interpolation schemes that own the weights
        // but not the distributions they are interpolating.

        // Distributions directly inheriting from AbsDistribution1D
        add_reader(ComparisonDistribution1D);
        add_reader(CompositeDistribution1D);
        add_reader(DistributionMix1D);
        add_reader(EdgeworthSeries1D);
        add_reader(InterpolatedDistro1D1P);
        add_reader(InterpolatedDistro1DNP);
        add_reader(LeftCensoredDistribution);
        add_reader(RatioOfNormals);
        add_reader(RightCensoredDistribution);
        add_reader(TransformedDistribution1D);
        add_reader(TruncatedDistribution1D);
        add_reader(FoldedDistribution1D);
    }
}
