#include "npstat/stat/KernelSensitivityCalculator.hh"

namespace npstat {
    void KernelSensitivityCalculator::makeLocalLegendreMatrix() const
    {
        if (!localLegendreMatrix_.nRows())
        {
            if (oracle_)
            {
                // Note that CdfFunctor1D below is Functor1<double,double>.
                // This means that "MultiplyByConst" will also produce
                // Functor1<double,double>. The normalization factor,
                // even though calculated originally in long double
                // precision, will be converted internally to the
                // type of Functor1 result, that is, just double.
                CdfFunctor1D cdf(*distro_);
                const long double normFactor = 1.0L/densityIntegral_;
                localLegendreMatrix_ = legendrePolyMatrix(
                    MultiplyByConst(cdf, normFactor), maxdegInputPoly_, xw_);
            }
            else
            {
                const unsigned nPoints = xw_.size();
                localLegendreMatrix_.resize(nPoints, maxdegInputPoly_);
                std::vector<long double> buf(maxdegInputPoly_ + 1U);
                ShiftedLegendreOrthoPoly1D poly;

                for (unsigned ipt=0; ipt<nPoints; ++ipt)
                {
                    const long double cdf = (ipt + 0.5L)/nPoints;
                    poly.allpoly(cdf, &buf[0], maxdegInputPoly_);
                    long double* row = localLegendreMatrix_.data() + maxdegInputPoly_*ipt;
                    const long double* from = &buf[1];
                    for (unsigned deg=1; deg<=maxdegInputPoly_; ++deg)
                        *row++ = *from++;
                }
            }
        }
    }
}
