#include <stdexcept>

#include "geners/binaryIO.hh"
#include "npstat/stat/LocScaleTransform1D.hh"

namespace npstat {
    void LocScaleTransform1D::validateScale(const double a)
    {
        if (a <= 0.0) throw std::invalid_argument(
            "In npstat::LocScaleTransform1D::validateScale: "
            "invalid scale parameter");
    }

    LocScaleTransform1D::LocScaleTransform1D(const double a, const double p)
        : AbsDistributionTransform1D(2U)
    {
        validateScale(p);
        params_[0] = a;
        params_[1] = p;
    }

    double LocScaleTransform1D::transformForward(const double x, double* dydx) const
    {
        if (dydx) *dydx = 1.0/params_[1];
        return (x - params_[0])/params_[1];
    }

    double LocScaleTransform1D::transformBack(const double y) const
    {
        return y*params_[1] + params_[0];
    }

    bool LocScaleTransform1D::isEqual(const AbsDistributionTransform1D& o) const
    {
        const LocScaleTransform1D& r = static_cast<const LocScaleTransform1D&>(o);
        return params_[0] == r.params_[0] && params_[1] == r.params_[1];
    }

    bool LocScaleTransform1D::write(std::ostream& os) const
    {
        gs::write_pod(os, params_[0]);
        gs::write_pod(os, params_[1]);
        return !os.fail();
    }

    LocScaleTransform1D* LocScaleTransform1D::read(const gs::ClassId& id,
                                                   std::istream& is)
    {
        static const gs::ClassId current(gs::ClassId::makeId<LocScaleTransform1D>());
        current.ensureSameId(id);

        double a, p;
        gs::read_pod(is, &a);
        gs::read_pod(is, &p);
        if (is.fail()) throw gs::IOReadFailure(
            "In npstat::LocScaleTransform1D::read: input stream failure");
        return new LocScaleTransform1D(a, p);
    }
}
