#ifndef NPSTAT_LOCSCALETRANSFORM1D_HH_
#define NPSTAT_LOCSCALETRANSFORM1D_HH_

/*!
// \file LocScaleTransform1D.hh
//
// \brief Transform y = (x - mu)/sigma
//
// Author: I. Volobouev
//
// October 2022
*/

#include "npstat/stat/AbsDistributionTransform1D.hh"

namespace npstat {
    class LocScaleTransform1D : public AbsDistributionTransform1D
    {
    public:
        LocScaleTransform1D(double mu, double sigma);
        inline virtual ~LocScaleTransform1D() {}

        inline virtual LocScaleTransform1D* clone() const
            {return new LocScaleTransform1D(*this);}

        double transformForward(double x, double* dydx) const;
        double transformBack(double y) const;
        inline bool isIncreasing() const {return true;}

        //@{
        /** Prototype needed for I/O */
        inline virtual gs::ClassId classId() const {return gs::ClassId(*this);}
        virtual bool write(std::ostream&) const;
        //@}

        static inline const char* classname()
            {return "npstat::LocScaleTransform1D";}
        static inline unsigned version() {return 1;}
        static LocScaleTransform1D* read(const gs::ClassId& id, std::istream& is);

    protected:
        virtual bool isEqual(const AbsDistributionTransform1D&) const;

    private:
        void validateScale(double sigma);

        inline void setParameterChecked(const unsigned which,
                                        const double value)
        {
            if (which)
                validateScale(value);
            params_[which] = value;
        }
        inline void setAllParametersChecked(const double* p)
        {
            validateScale(p[1]);
            params_[0] = p[0];
            params_[1] = p[1];
        }
        inline double getParameterChecked(const unsigned which) const
            {return params_[which];}

        double params_[2];
    };
}

#endif // NPSTAT_LOCSCALETRANSFORM1D_HH_
