#ifndef NPSTAT_LOGNORMALTRANSFORM1D_HH_
#define NPSTAT_LOGNORMALTRANSFORM1D_HH_

/*!
// \file LognormalTransform1D.hh
//
// \brief Transform used to generate the lognormal distribution
//
// Author: I. Volobouev
//
// November 2022
*/

#include "npstat/stat/AbsDistributionTransform1D.hh"

namespace npstat {
    class LognormalTransform1D : public AbsDistributionTransform1D
    {
    public:
        LognormalTransform1D(double delta, double gamma,
                             double xi, bool positiveSkewness);
        inline virtual ~LognormalTransform1D() {}

        inline virtual LognormalTransform1D* clone() const
            {return new LognormalTransform1D(*this);}

        double transformForward(double x, double* dydx) const;
        double transformBack(double y) const;
        bool isIncreasing() const;

        //@{
        /** Prototype needed for I/O */
        inline virtual gs::ClassId classId() const {return gs::ClassId(*this);}
        virtual bool write(std::ostream&) const;
        //@}

        static inline const char* classname()
            {return "npstat::LognormalTransform1D";}
        static inline unsigned version() {return 1;}
        static LognormalTransform1D* read(const gs::ClassId& id, std::istream& is);

    protected:
        virtual bool isEqual(const AbsDistributionTransform1D&) const;

    private:
        inline void setParameterChecked(const unsigned which, const double value)
            {params_[which] = value;}
        inline void setAllParametersChecked(const double* p)
            {for (unsigned i=0; i<4U; ++i) {params_[i] = p[i];}}
        inline double getParameterChecked(const unsigned which) const
            {return params_[which];}

        double params_[4];
    };
}

#endif // NPSTAT_LOGNORMALTRANSFORM1D_HH_
