#ifndef NPSTAT_PLOGLIOSDE1DRESULT_HH_
#define NPSTAT_PLOGLIOSDE1DRESULT_HH_

/*!
// \file PLogliOSDE1DResult.hh
//
// \brief Class for representing results of the
//        pseudo-log-likelihood OSDE optimization algorithms
//
// Author: I. Volobouev
//
// March 2023
*/

#include <vector>

namespace npstat {
    enum PLogliOSDE1DMinimizerCode {
        PLOSDE_NOT_OPTIMIZED = 0,
        PLOSDE_MINUIT,
        PLOSDE_N_MINIMIZER_CODES
    };

    const char* plOSDEMinimizerName(unsigned minimizerCode);

    class PLogliOSDE1DResult
    {
    public:
        inline PLogliOSDE1DResult(const std::vector<double>& i_coeffs,
                                  const double i_pseudoLogli,
                                  const double i_maxAbsGradient,
                                  const unsigned i_minimizerCode,
                                  const unsigned i_nIterations,
                                  const bool i_converged,
                                  const bool i_badHessian)
            : coeffs_(i_coeffs),
              pseudoLogli_(i_pseudoLogli),
              maxAbsGradient_(i_maxAbsGradient),
              minimizerCode_(i_minimizerCode),
              nIterations_(i_nIterations),
              converged_(i_converged),
              badHessian_(i_badHessian)
        {
        }                          

        inline void setMinimizerCode(const unsigned c) {minimizerCode_ = c;}

        inline const std::vector<double>& coeffs() const {return coeffs_;}
        inline double pseudoLogli() const {return pseudoLogli_;}
        inline double maxAbsGradient() const {return maxAbsGradient_;}
        inline unsigned minimizerCode() const {return minimizerCode_;}
        inline unsigned nIterations() const {return nIterations_;}
        inline bool converged() const {return converged_;}
        inline bool badHessian() const {return badHessian_;}

        inline bool operator<(const PLogliOSDE1DResult& r) const
        {
            if (r.pseudoLogli_ < pseudoLogli_) return true;
            if (pseudoLogli_ < r.pseudoLogli_) return false;
            if (maxAbsGradient_ < r.maxAbsGradient_) return true;
            if (r.maxAbsGradient_ < maxAbsGradient_) return false;
            if (converged_ && !r.converged_) return true;
            if (!converged_ && r.converged_) return false;
            return false;
        }

    private:
        std::vector<double> coeffs_;
        double pseudoLogli_;
        double maxAbsGradient_;
        unsigned minimizerCode_;
        unsigned nIterations_;
        bool converged_;
        bool badHessian_;
    };
}

#endif // NPSTAT_PLOGLIOSDE1DRESULT_HH_
