#ifndef NPSTAT_SINEGOFTEST1D_HH_
#define NPSTAT_SINEGOFTEST1D_HH_

/*!
// \file SineGOFTest1D.hh
//
// \brief Goodness-of-fit test based on the sine expansion of the
//        difference between the empirical cdf and the cdf of the null
//
// Note that the sine expansion of this difference is equivalent to
// the cosine expansion of the empirical comparison density
//
// Author: I. Volobouev
//
// November 2022
*/

#include "npstat/stat/AbsSmoothGOFTest1D.hh"

namespace npstat {
    class SineGOFTest1D : public AbsSmoothGOFTest1D
    {
    public:
        // The mask, if provided, should contain nSineTerms elements.
        // Put some number in the mask in order to discard the
        // corresponding eigenvector and put 0 there to keep it.
        // The sine terms (sin(Pi n x)) numbering in the mask
        // is shifted: n = 1 corresponds to mask[0], n = 2 corresponds
        // to mask[1], etc.
        inline SineGOFTest1D(const AbsDistribution1D& distro,
                             const unsigned nSineTerms,
                             const unsigned char* mask=0,
                             const unsigned lenMask=0)
            : AbsSmoothGOFTest1D(distro, nSineTerms, mask, lenMask) {}

        inline SineGOFTest1D(const AbsDistribution1D& distro,
                             const std::vector<int>& mask)
            : AbsSmoothGOFTest1D(distro, mask) {}

        inline virtual ~SineGOFTest1D() {}

        inline virtual std::string shortName() const
            {return testNameWithMask("Sine");}

        virtual unsigned normalizedDeviations(
            const double* data, unsigned long sz,
            bool isDataSorted,
            double* deviations, unsigned lenDeviations) const;

        virtual unsigned normalizedDeviations(
            const float* data, unsigned long sz,
            bool isDataSorted,
            double* deviations, unsigned lenDeviations) const;

    private:
        template<typename Numeric>
        unsigned normalizedDeviationsHelper(
            const Numeric* data, unsigned long lenData,
            double* deviations) const;
    };
}

#include "npstat/stat/SineGOFTest1D.icc"

#endif // NPSTAT_SINEGOFTEST1D_HH_
