#include <cmath>
#include <vector>
#include <cassert>

#include "npstat/nm/SimpleScalarProduct.hh"
#include "npstat/nm/shiftedLegendreIntegral.hh"

#include "npstat/stat/directedCDTestCumulants.hh"

namespace npstat {
    std::pair<double,double> directedCDTestCumulants(
        const double* i_direction, const unsigned nTerms,
        const unsigned sampleSize, const bool memoize)
    {
        assert(i_direction);
        assert(nTerms);
        assert(sampleSize);

        // Construct the normalized direction vector
        std::vector<long double> dirBuf(i_direction, i_direction+nTerms);
        {
            const SimpleScalarProduct<long double> sp;
            long double* dir = &dirBuf[0];
            const long double sumsq = sp(dir, dir, nTerms);
            assert(sumsq > 0.0L);
            const long double norm = sqrtl(sumsq);
            for (unsigned i=0; i<nTerms; ++i)
                dir[i] /= norm;
        }
        const long double* dir = &dirBuf[0];

        long double skewSum = 0.0L, kurtSum = 0.0L;
        for (unsigned i=0; i<nTerms; ++i)
        {
            const long double iProd = dir[i];
            for (unsigned j=0; j<nTerms; ++j)
            {
                const long double jProd = iProd*dir[j];
                for (unsigned k=0; k<nTerms; ++k)
                {
                    const long double kProd = jProd*dir[k];
                    skewSum += kProd*shiftedLegendreIntegral(i+1U, j+1U, k+1U);

                    for (unsigned l=0; l<nTerms; ++l)
                        kurtSum += kProd*dir[l]*shiftedLegendreIntegral(
                            i+1U, j+1U, k+1U, l+1U, memoize);
                }
            }
        }

        return std::pair<double,double>(skewSum/sqrtl(sampleSize),
                                        (kurtSum - 3.0L)/sampleSize);
    }
}
