#ifndef NPSTAT_DIRECTEDCDTESTCUMULANTS_HH_
#define NPSTAT_DIRECTEDCDTESTCUMULANTS_HH_

/*!
// \file directedCDTestCumulants.hh
//
// \brief The 3rd and 4th cumulants for the distribution of the
//        statistic for the directed comparison density test based on
//        shifted Legendre polynomials under the null hypothesis
//
// Author: I. Volobouev
//
// July 2023
*/

#include <utility>

namespace npstat {
    // Note that the time needed to calculate these cumulants
    // scales as O(nTerms^4). Memoization of intermediate results
    // might help for large "nTerms" -- you should time your code both
    // with and without memoization and decide whether you want to
    // use it.
    //
    // The input direction array does not have to be normalized --
    // it will be normalized internally. It is assumed that the
    // first array element is the direction component for the
    // polynomial of degree 1, the second element corresponds
    // to degree 2, etc.
    //
    // In the returned pair, the first element is the skewness (the
    // 3rd cumulant, as the variance is 1) and the second element
    // is the excess kurtosis (the 4th cumulant).
    //
    std::pair<double,double> directedCDTestCumulants(
        const double* direction, unsigned nTerms,
        unsigned sampleSize, bool memoize = false);
}

#endif // NPSTAT_DIRECTEDCDTESTCUMULANTS_HH_
