#ifndef NPSTAT_JOHNSONINTEGRAL_HH_
#define NPSTAT_JOHNSONINTEGRAL_HH_

/*!
// \file johnsonIntegral.hh
//
// \brief Integrate a function multiplied by one of the Johnson curves
//
// Author: I. Volobouev
//
// November 2022
*/

#include "npstat/nm/SimpleFunctors.hh"
#include "npstat/stat/AbsDistribution1D.hh"

namespace npstat {
    // The integration is performed using a transformation which
    // converts the Johnson curve into the standard normal. Then
    // the Gauss-Hermite quadrature is used. "nIntegrationPoints"
    // must be supported by the "GaussHermiteQuadrature" class.
    double johnsonIntegral(const AbsDistribution1D& johnsonCurve,
                           const Functor1<double,double>& fcn,
                           unsigned nIntegrationPoints);
}

#endif // NPSTAT_JOHNSONINTEGRAL_HH_
