#ifndef NPSTAT_JOHNSONTRANSFORM_HH_
#define NPSTAT_JOHNSONTRANSFORM_HH_

/*!
// \file johnsonTransform.hh
//
// \brief Determine the transform that generates a member of Johnson family
//        of frequency curves
//
// Author: I. Volobouev
//
// October 2022
*/

#include <memory>

#include "npstat/stat/AbsDistribution1D.hh"
#include "npstat/stat/AbsDistributionTransform1D.hh"

namespace npstat {
    // This function will throw an exception if the input
    // distribution is not a member of the Johnson family and not
    // a TransformedDistribution1D using the standard normal
    std::unique_ptr<AbsDistributionTransform1D> johnsonTransform(
        const AbsDistribution1D& distro);

#ifdef SWIG
    inline AbsDistributionTransform1D* johnsonTransform2(
        const AbsDistribution1D& d)
    {
        std::unique_ptr<AbsDistributionTransform1D> ptr = johnsonTransform(d);
        return ptr.release();
    }
#endif // SWIG
}

#endif // NPSTAT_JOHNSONTRANSFORM_HH_
