#include "npstat/nm/SimpleScalarProduct.hh"
#include "npstat/nm/modifiedGramSchmidt.hh"

#include "npstat/stat/Distributions1D.hh"
#include "npstat/stat/randomOrthogonalMatrix.hh"

namespace npstat {
    npstat::Matrix<double> randomOrthogonalMatrix(
        npstat::AbsRandomGenerator& g, const unsigned dim)
    {
        // For orthonormalization purposes, it is useful
        // to switch to a higher precision type
        Matrix<long double> m(dim, dim);
        long double* data = m.data();
        const Gauss1D gauss(0.0, 1.0);
        double tmp;

        const unsigned long len = m.length();
        for (unsigned long i=0; i<len; ++i)
        {
            gauss.random(g, &tmp);
            *data++ = tmp;
        }
        modifiedGramSchmidt(m.data(), dim, dim,
                            SimpleScalarProduct<long double>());
        return m.T();
    }
}
