#include <stdexcept>

#include "npstat/stat/saddlepointDistribution1D.hh"

namespace npstat {
    CPP11_auto_ptr<Tabulated1D> saddlepointDistribution1D(
        const AbsCGF1D& cgf, const unsigned approximationOrder,
        const unsigned interpolationDegree, unsigned nCoords,
        const double xmin, const double xmax)
    {
        if (!(xmin < xmax)) throw std::invalid_argument(
            "In npstat::saddlepointDistribution1D: invalid support interval");
        if (nCoords < 2U)
            nCoords = 2U;

        const double range = xmax - xmin;
        const unsigned ncM1 = nCoords - 1U;
        const double step = range/ncM1;
        std::vector<double> density(nCoords);
        for (unsigned i=0; i<ncM1; ++i)
            density[i] = cgf.saddlepointDensityApprox(
                approximationOrder, xmin + i*step);
        density[ncM1] = cgf.saddlepointDensityApprox(approximationOrder, xmax);
        return CPP11_auto_ptr<Tabulated1D>(new Tabulated1D(
                   xmin, range, density, interpolationDegree));
    }
}
