#ifndef NPSTAT_PRECISESUM_HH_
#define NPSTAT_PRECISESUM_HH_

#include <cmath>
#include <cfloat>
#include <vector>
#include <cassert>
#include <utility>
#include <algorithm>

namespace npstat {
    template<typename Numeric>
    inline double preciseSum(const Numeric* data, const unsigned dataLen)
    {
        if (!dataLen)
            return 0.0;
        assert(data);

        std::vector<std::pair<Numeric,Numeric> > tmp(dataLen);
        for (unsigned i=0; i<dataLen; ++i)
        {
            tmp[i].first = std::abs(data[i]);
            tmp[i].second = data[i];
        }
        std::sort(tmp.begin(), tmp.end());
        long double sum = 0.0L;
        for (unsigned i=0; i<dataLen; ++i)
            sum += tmp[i].second;
        return sum;
    }
}

#endif // NPSTAT_PRECISESUM_HH_
